#!/bin/bash
# vim: set sts=4 sw=4 et tw=0 :
#

set -e

TESTDIR=$(cd $(dirname $0); pwd; cd - &>/dev/null)
. "${TESTDIR}/config.sh"

#########
# Setup #
#########
trap "setup_failure" ERR

# For example, test_profiles_are_applied() has problems if you run it as
# root because programs behave differently if you run them as root instead
# of a normal user.
check_not_root

setup_success

###########
# Execute #
###########

_watch_audit_logs() {
    sudo tail -n 0 -f "${AUDIT_LOG}" > "$1"
}

# Check whether apparmor is enabled
test_apparmor_enabled() {
    [[ -d /etc/apparmor.d ]] || return 1
    sudo aa-status --enabled || return 1
}

# Check if profiles are being parsed correctly
test_profile_parsing() {
    local ret=0
    if ! /sbin/apparmor_parser -Q "${RESOURCE_DIR}/apparmor/test-profiles/usr.lib.valid-profile"; then
        whine "Failed to parse a valid profile!"
        ret=1
    fi

    if /sbin/apparmor_parser -Q "${RESOURCE_DIR}/apparmor/test-profiles/usr.lib.invalid-profile"; then
        whine "Successfully parsed an invalid profile!"
        ret=1
    fi

    return $ret
}

# Check if profiles have any syntax errors
test_profile_syntax() {
    local i ret=0
    for i in "${APPARMOR_PROFILES_DIRS[@]}"; do
        if ! [[ -d $i ]]; then
            whine "Invalid apparmor profile directory: $i"
        fi
    done
    for i in $(find "${APPARMOR_PROFILES_DIRS[@]}" -maxdepth 1 -type f); do
        if [[ -e "$i" ]] && sudo apparmor_parser -Q "$i"; then
            :
        else
            whine "Failed to parse $i"
            ret=1
        fi
    done
    return $ret
}

# Check whether all the profiles are in complain mode
test_profiles_complain_mode() {
    PROFILED="$(sudo aa-status --profiled)"
    COMPLAINING="$(sudo aa-status --complaining)"
    if [[ ${PROFILED} != ${COMPLAINING} ]]; then
	whine "Not all profiles are in complain mode (${COMPLAINING}/${PROFILED})"
	return 1
    fi

    return 0
}

# Check whether profiles are being applied properly by comparing the audit 
# accesses against a "good" audit log for the same run
test_profiles_are_applied() {
    local each workdir
    workdir="$(create_temp_workdir)"
    for each in "${workdir}/profile-auditing"/*; do
        ## Run, and store the logs
        _watch_audit_logs "$each/audit.log" &
        $(<$each/run)
        _sleep 3 && kill %1 && sync
        ## Extract logs
        # This hacky way of passing information via environment variables is a
        # temporary measure till we add option parsing to aa_log_extract_tokens.pl
        AA_PROFILE_WANTED=$(<$each/aa-profile-wanted) aa_log_extract_tokens.pl AUDITING > "$each/filtered-audit.log" < "$each/audit.log"
        sync && _sleep 1
        ## Compare logs
        # For now, we do a straight compare, completely ignoring the 
        # possibility of out-of-order resource access. Later, we should write
        # a small function to do a set-compare to ensure that the same resources
        # are accessed each time, but ignoring the order.
        diff -u "$each/log" "$each/filtered-audit.log"
    done
}

trap "test_failure" ERR

src_test_pass <<-EOF
test_apparmor_enabled
test_profile_parsing
test_profile_syntax
#test_profiles_complain_mode
#test_profiles_are_applied
EOF

test_success
