/* vim: set sts=2 sw=2 et :
 *
 * File/Directory metadata change monitoring using Grilo
 *
 */

/* Don't look at me like that. It's fine. */
#include "grilo-do-metadata-changes.c"

#include "util.h"

#define GRL_LOG_DOMAIN_DEFAULT  example_log_domain
GRL_LOG_DOMAIN_STATIC(example_log_domain);

#define BROWSE_CHUNK_SIZE 10
#define TOPLEVEL_DIR "toplevel"

static const gchar *chosen_test_path;

static gchar*
get_grl_change_type_str (GrlSourceChangeType change_type)
{
  if (change_type == GRL_CONTENT_CHANGED)
    return "GRL_CONTENT_CHANGED";
  if (change_type == GRL_CONTENT_ADDED)
    return "GRL_CONTENT_ADDED";
  if (change_type == GRL_CONTENT_REMOVED)
    return "GRL_CONTENT_REMOVED";
}

static void
change_handler_cb (gpointer data,
                   gpointer user_data)
{
  const gchar *change_path;
  GrlMedia *media = GRL_MEDIA (data);
  GriloTestAction *action = (GriloTestAction*) user_data;

  change_path = g_filename_from_uri (grl_media_get_url (GRL_MEDIA (data)), NULL, NULL);

  if (g_strcmp0 (action->change_path, change_path) != 0)
    g_error ("Expected change path %s, got change path %s",
             action->change_path, change_path);
  g_debug ("Yey! %s == %s!", action->change_path, change_path);
  actions = g_slist_delete_link (actions, actions);
}

static void
content_changed_cb (GrlSource                 *source,
                    GPtrArray                 *changed_medias,
                    GrlSourceChangeType        change_type,
                    gboolean                   location_unknown,
                    gboolean                   user_data)
{
  GriloTestAction *action;
  g_debug ("Content changed!");

  g_debug ("Is stack fresh?");
  while (!stack_fresh)
    g_thread_yield ();
  g_debug ("Stack is fresh! Time to make it stale");

  g_mutex_lock (&mutex);
  action = g_slist_nth_data (actions, 1);
  if (action->change_type != change_type)
    g_error ("Expected change type %s, got change type %s",
             get_grl_change_type_str (action->change_type),
             get_grl_change_type_str (change_type));

  g_ptr_array_foreach (changed_medias, change_handler_cb, action);
  stack_fresh = FALSE;
  g_mutex_unlock (&mutex);
}

static void
source_monitor (gpointer data,
                gpointer user_data)
{
  GrlSource *source = GRL_SOURCE (data);
  GrlMedia *media = GRL_MEDIA (user_data);
  GError *error = NULL;
  const gchar *source_name;
  
  source_name = grl_source_get_name (source);

  if (!(grl_source_supported_operations (source) & GRL_OP_NOTIFY_CHANGE)) {
    g_warning ("Source '%s' doesn't support GRL_OP_NOTIFY_CHANGE", source_name);
    return;
  }

  g_debug ("Telling source '%s' to start notifying", source_name);
  if (!grl_source_notify_change_start (GRL_SOURCE (source), NULL))
    g_critical ("Source '%s' can't start notifying!", source_name);

  g_signal_connect (GRL_SOURCE (source), "content-changed",
                    G_CALLBACK (content_changed_cb), NULL);

  thread = g_thread_new ("do-changes", (GThreadFunc) begin_monitor_testing, NULL);
}

static void
load_plugins (void)
{
  GrlRegistry *registry;
  GrlSource *source;
  GError *error = NULL;

  registry = grl_registry_get_default ();

  /* Load plugin */
  if (!grl_registry_load_plugin_by_id (registry, "grl-filesystem", &error))
    g_error ("Failed to load plugin: %s", error->message);

  source = grl_registry_lookup_source (registry, "grl-filesystem");
  if (!source)
    g_error ("Unable to load grl-filesystem plugin");

  source_monitor (source, NULL);
}

gint
main (int     argc,
      gchar  *argv[])
{
  GFile *toplevel_dir;
  GError *error = NULL;

  grl_init (&argc, &argv);
  GRL_LOG_DOMAIN_INIT (example_log_domain, "example");

  if (argc != 2) {
    g_printf ("Usage: %s <path to monitor>\n", argv[0]);
    return 1;
  }

  actions = g_slist_alloc ();
  chosen_test_path = argv[1];
  toplevel_path = g_build_filename (chosen_test_path, TOPLEVEL_DIR, NULL);
  toplevel_dir = g_file_new_for_commandline_arg (toplevel_path);
  if (!g_file_make_directory_with_parents (toplevel_dir, NULL, &error))
    g_error ("Unable to create directory %s: %s\n",
             toplevel_path, error->message);

  g_object_unref (toplevel_dir);

  chaiwala_grilo_config_plugins (chosen_test_path);
  load_plugins ();

  /* Run the main loop */
  loop = g_main_loop_new (NULL, FALSE);
  g_main_loop_run (loop);
  return 0;
}

