/* vim: set sts=2 sw=2 et : */ 
/*
 * Browsing in Grilo.
 * Shows the first BROWSE_CHUNK_SIZE elements of each browsable source
 *
 * XXX: No pagination yet! See grilo-test-ui. It's somewhat complicated.
 */

#include <grilo.h>

#define GRL_LOG_DOMAIN_DEFAULT  example_log_domain
GRL_LOG_DOMAIN_STATIC(example_log_domain);

#define BROWSE_CHUNK_SIZE 10

static void source_browser (gpointer data,
                            gpointer user_data);
static void element_browser (gpointer data,
                             gpointer user_data);

/* The grl-tracker plugin returns no sources if it's loaded synchronously.
 * But we need a test program that runs and exits without interaction.
 * So, we use a global loop and quit once we're done. */
static GMainLoop *loop;
/* Once Tracker source is loaded we can quit */
static gint sources_count = 0;

static void
element_browser (gpointer data,
                 gpointer user_data)
{
  GrlMedia *media = GRL_MEDIA (data);
  GrlSource *source = GRL_SOURCE (user_data);

  /* Check if we got a valid media object as some plugins may call the callback
     with a NULL media under certain circumstances (for example when they
     cannot estimate the number of remaining results and they find suddenly they
     don't have any more results to send) */
  if (!media) {
    g_debug ("Media element is NULL!");
    goto out;
  }
  
  const gchar *title = grl_media_get_title (media);

  /* If the media is a container (box), that means we will browse it again */
  if (GRL_IS_MEDIA_BOX (media)) {
    guint childcount = grl_media_box_get_childcount (GRL_MEDIA_BOX (media));
    g_debug ("\t Got '%s' (container with %d elements)", title, childcount);

    source_browser (source, media);
  } else {
    const gchar *url = grl_media_get_url (media);
    // Thumbnail fetching isn't working? Unsure.
    const gchar *thumbnail = grl_media_get_thumbnail (media);
    if (GRL_IS_MEDIA_AUDIO (media)) {
      const gchar *artist = grl_media_audio_get_artist (GRL_MEDIA_AUDIO (media));
      const gchar *album = grl_media_audio_get_album (GRL_MEDIA_AUDIO (media));
      g_printf ("\t Got '%s' by the artist '%s' from the album '%s'\n",
               title, artist, album);
    } else {
      g_printf ("\t Got '%s'\n", title);
    }
    g_printf ("\t\t URL: %s\n", url);
    g_printf ("\t\t Thumbnail: %s\n", thumbnail);
  }

out:
  g_object_unref (media);
}

static void
source_browser (gpointer data,
                gpointer user_data)
{
  GrlSource *source = GRL_SOURCE (data);
  GrlMedia *media = GRL_MEDIA (user_data);
  GList *media_elements;
  GError *error = NULL;
  GList *keys;
  GrlOperationOptions *options;
  GrlCaps *caps;

  keys = grl_metadata_key_list_new (GRL_METADATA_KEY_TITLE,
                                    GRL_METADATA_KEY_ALBUM,
                                    GRL_METADATA_KEY_ARTIST,
                                    GRL_METADATA_KEY_THUMBNAIL,
                                    GRL_METADATA_KEY_URL,
                                    GRL_METADATA_KEY_CHILDCOUNT,
                                    NULL);

  g_debug ("Detected new source available: '%s'",
	   grl_source_get_name (source));

  if (!(grl_source_supported_operations (source) & GRL_OP_BROWSE))
    goto out;

  g_debug ("Browsing source: %s", grl_source_get_name (source));
  /* Here is how you can browse a source, you have to provide:
     1) The source you want to browse contents from.
     2) The container object you want to browse (NULL for the root container)
     3) A list of metadata keys we are interested in.
     4) Options to control certain aspects of the browse operation.
     5) A callback that the framework will invoke for each available result
     6) User data for the callback
     It returns an operation identifier that you can use to match results
     with the corresponding request (we ignore it here) */

  caps = grl_source_get_caps (source, GRL_OP_BROWSE);
  options = grl_operation_options_new (caps);
  grl_operation_options_set_count (options, BROWSE_CHUNK_SIZE);
  grl_operation_options_set_flags (options, GRL_RESOLVE_IDLE_RELAY);

  media_elements = grl_source_browse_sync (GRL_SOURCE (source),
                                           media, keys,
                                           options,
                                           &error);
  if (!media_elements) {
    g_debug ("No elements found for source: %s!",
             grl_source_get_name (source));
    goto out;
  }

  if (error)
    g_error ("Failed to browse source: %s", error->message);

  g_list_foreach (media_elements, element_browser, source);

out:
  g_list_free (keys);
}

static void
source_added_cb (GrlRegistry *registry, gpointer user_data)
{
  GrlSource *source = GRL_SOURCE (user_data);
  g_debug ("Detected new source available: '%s'",
           grl_source_get_name (source));
  sources_count++;

  source_browser (source, NULL);

  if (sources_count > 0)
    g_main_loop_quit (loop);
}

static void
load_plugins (void)
{
  GrlRegistry *registry;
  GError *error = NULL;

  registry = grl_registry_get_default ();

  if (!grl_registry_load_plugin_by_id (registry, "grl-tracker", &error))
    g_error ("Failed to load tracker plugin: %s", error->message);
  
  g_signal_connect (registry, "source-added",
                    G_CALLBACK (source_added_cb), NULL);
}

gint
main (int argc, gchar *argv[])
{
  grl_init (&argc, &argv);
  GRL_LOG_DOMAIN_INIT (example_log_domain, "example");
  load_plugins ();
  /* Run the main loop */
  loop = g_main_loop_new (NULL, FALSE);
  g_main_loop_run (loop);
  return 0;
}

