#!/bin/bash
# vim: set sts=4 sw=4 et tw=0 :
#

set -e

TESTDIR=$(cd $(dirname $0); pwd; cd - &>/dev/null)

. "${TESTDIR}/config.sh"
. "${TESTDIR}/../../common/tracker-utils.sh"
. "${TESTDIR}/../../common/thumbnailer-utils.sh"

#########
# Setup #
#########
configure_tracker() {
    _reset_tracker_config
    # Disable all indexing
    set_gsettings_key "${TRACKER_MINER_SCHEMA}" index-recursive-directories "[]"
    set_gsettings_key "${TRACKER_MINER_SCHEMA}" index-single-directories "[]"
    # Never wait before starting to mine
    set_gsettings_key "${TRACKER_MINER_SCHEMA}" initial-sleep 0
    set_gsettings_key "${TRACKER_MINER_SCHEMA}" sched-idle never
}

trap "setup_failure" ERR

set -x
# Need a DBus session for this test
ensure_dbus_session

configure_tracker
set +x

setup_success

###########
# Execute #
###########
TRACKER_MINER_LOG="${WORKDIR}/tracker-miner.log"

_reset_index_config() {
    # Wipe the database so that we don't get unpredictable query replies
    tracker-control -r

    # Kill all tracker processes. Do this via systemctl rather than
    # tracker-control to avoid systemd respawning the process.
    tracker-control -t
    systemctl --user stop tracker-store tracker-miner-fs

    # Set indexing configuration
    if [[ -n "$1" ]]; then
        set_gsettings_key "${TRACKER_MINER_SCHEMA}" index-single-directories "['${1}']"
    else
        set_gsettings_key "${TRACKER_MINER_SCHEMA}" index-single-directories "[]"
    fi
    if [[ -n "$2" ]]; then
        set_gsettings_key "${TRACKER_MINER_SCHEMA}" index-recursive-directories "['${2}']"
    else
        set_gsettings_key "${TRACKER_MINER_SCHEMA}" index-recursive-directories "[]"
    fi
}

_test_indexing() {
    set -x
    local ret=0
    local tempdir=$(create_temp_workdir)
    local indexdir="${MEDIA_RESOURCE_DIR}/$1"

    # Kill all tracker processes, reset configuration
    _reset_index_config "${indexdir}"

    _do_initial_indexing

    _check_args_indexed "${indexdir}"/* || ret=1
    _kill_monitor
    _reset_tracker_config
    set +x
    return $ret
}

test_recursive_indexing() {
    set -x
    local i
    local ret=0
    local tempdir=$(create_temp_workdir)
    local indexdir="${MEDIA_RESOURCE_DIR}"
    local uris=()

    # Kill all tracker processes, reset configuration
    _reset_index_config "" "${indexdir}"

    _do_initial_indexing

    _check_args_indexed "${indexdir}/"*/* || ret=1
    _check_args_playable "${indexdir}/"videos/* "${indexdir}/"audio/*.{flac,wav} || ret=1

    for i in "${indexdir}/"*/*.{jpg,pdf,odt,ods,odp,png,ogv}; do
        uris+=("file://$i")
    done
    _check_uris_have_thumbnail normal $uris || ret=1

    _kill_monitor
    _reset_tracker_config
    set +x
    return $ret
}

test_audio_indexing() {
    _test_indexing audio
}

test_playlist_indexing() {
    _test_indexing playlists
}

test_video_indexing() {
    _test_indexing videos
}

test_image_indexing() {
    _test_indexing images
}

test_document_indexing() {
    _test_indexing documents
}

test_playlist_contents_indexing(){
    set -x
    local i pos ret=0 count=0 playlist_entries=()
    local tempdir=$(create_temp_workdir)
    local playlist="Generic_Sounds.pls"

    # Kill all tracker processes, reset configuration
    _reset_index_config "" "${tempdir}"

    # Copy data to be indexed because we need a specific subset of it
    src_copy "${MEDIA_RESOURCE_DIR}/playlists/${playlist}" "${tempdir}"
    src_copy "${MEDIA_RESOURCE_DIR}/audio/" "${tempdir}"
    _do_initial_indexing

    playlist_entries=($(_get_tracker_info_values "${tempdir}/${playlist}" \
                        "nfo:hasMediaFileListEntry"))

    # Check if playlist contents were indexed
    if [[ "${#playlist_entries[@]}" == 0 ]]; then
        whine "Tracker didn't index the playlist's contents"
        ret=1
    fi

    # Check if playlist contents match
    for i in "${playlist_entries[@]}"; do
        : $((count++))
        pos=$(_get_tracker_info_values "${i}" "nfo:listPosition")
        if [[ "$pos" != "$count.0" ]]; then
            whine "Tracker didn't extract the listPosition properly!"
            ret=1
        fi
    done

    _kill_monitor
    _reset_tracker_config
    set +x
    return $ret
}

test_journal_disabled() {
    set -x
    local ret=0
    # Kill all tracker processes. Do this via systemctl rather than
    # tracker-control to avoid systemd respawning the process.
    tracker-control -t
    systemctl --user stop tracker-store tracker-miner-fs

    # Restart mining
    tracker-control -s

    # Is the journal around?
    [[ ! -f "${XDG_DATA_HOME:-${HOME}/.local/share}"/tracker/data/tracker-store.journal ]] || ret=1
    set +x
    return $ret
}

test_monitoring_add() {
    set -x
    local ret=0
    local tempdir=$(create_temp_workdir)

    # Kill all tracker processes, reset configuration
    _reset_index_config "${tempdir}"

    # Add files!
    src_copy_contents "${MEDIA_RESOURCE_DIR}/images" "${tempdir}" 2>&1
    _do_initial_indexing

    # Add files again!
    src_copy_contents "${MEDIA_RESOURCE_DIR}/documents" "${tempdir}" 2>&1
    sync

    # Wait for reindexing to complete
    _spin_on_indexing_restart
    _spin_on_indexing_finish

    _check_args_indexed "${tempdir}"/* || ret=1
    _kill_monitor
    _reset_tracker_config
    set +x
    return $ret
}

test_monitoring_delete() {
    set -x
    local i pdf_files ret=0
    local tempdir=$(create_temp_workdir)

    # Kill all tracker processes, reset configuration
    _reset_index_config "${tempdir}"

    # Add document files!
    src_copy_contents "${MEDIA_RESOURCE_DIR}/documents" "${tempdir}" 2>&1
    _do_initial_indexing

    # Delete pdf files!
    for i in "${tempdir}"/*.pdf; do
        pdf_files+=($i)
    done
    rm -rvf "${pdf_files[@]}"
    sync

    # Wait for indexing to complete
    _spin_on_indexing_restart
    _spin_on_indexing_finish

    _check_args_indexed "${tempdir}/"* || ret=1
    _check_args_not_indexed "${pdf_files[@]}" || ret=1
    _kill_monitor
    _reset_tracker_config
    set +x
    return $ret
}

test_monitoring_modify() {
    set -x
    local ret=0
    local mod_file="lorem_text.txt"
    local old_filesize new_filesize
    local tempdir=$(create_temp_workdir)

    # Kill all tracker processes, reset configuration
    _reset_index_config "${tempdir}"

    # Add document files!
    src_copy "${MEDIA_RESOURCE_DIR}/documents/${mod_file}" "${tempdir}"
    _do_initial_indexing

    old_filesize=$(_get_tracker_info_values "${tempdir}/${mod_file}" "nfo:fileSize")

    # Modify file
    echo "NEW TEXT SOMETHING SOMETHING" >> "${tempdir}/${mod_file}"
    # Sync after writing to file
    sync

    # Wait for reindexing to complete
    _spin_on_indexing_restart
    _spin_on_indexing_finish

    new_filesize=$(_get_tracker_info_values "${tempdir}/${mod_file}" "nfo:fileSize")
    [[ ${new_filesize} -gt ${old_filesize} ]] || ret=1
    _kill_monitor
    _reset_tracker_config
    set +x
    return $ret
}

# Get the Media content URI using specific Album/Artist/Title query terms
test_query_metadata_info() {
    set -x
    local ret=0
    local tempdir=$(create_temp_workdir)

    # Kill all tracker processes, reset configuration
    _reset_index_config "${tempdir}"

    # Add tagged audio files!
    src_copy "${MEDIA_RESOURCE_DIR}/audio/"*.{oga,mp3,flac} "${tempdir}"
    _do_initial_indexing

    if ! _check_args_indexed "${tempdir}"/*; then
        whine "Audio files didn't even get indexed!"
        _kill_monitor
        _reset_tracker_config
        set +x
        return 1
    fi

    if ! [[ $("${TESTLIBDIR}/tracker-metadata-search" --album "GNOME Audio" | wc -l) -eq 4 ]]; then
        whine "Album search failed!"
        ret=1
    elif ! [[ $("${TESTLIBDIR}/tracker-metadata-search" --artist "Conrad Parker" | wc -l) -eq 4 ]]; then
        whine "Artist search failed!"
        ret=1
    elif ! [[ $("${TESTLIBDIR}/tracker-metadata-search" --title "GNOME Generic Sound" | wc -l) -eq 4 ]]; then
        whine "Title search failed!"
        ret=1
    fi
    _kill_monitor
    _reset_tracker_config
    set +x
    return $ret
}

test_grilo_local_storage_tracker() {
    set -x
    local ret=0
    local browsed_files_log="${WORKDIR}/browsed-files.log"
    local browse_output_log="${WORKDIR}/browse-output.log"
    local expected_files_log="${WORKDIR}/expected-files.log"
    local expected_output_string="Got 'GNOME Generic Sound' by the artist 'Conrad Parker' from the album 'GNOME Audio'"
    local tempdir=$(create_temp_workdir)

    # Kill all tracker processes, reset configuration
    _reset_index_config "" "${tempdir}"

    # Copy data to be indexed
    src_copy "${MEDIA_RESOURCE_DIR}/"{audio,videos,images} "${tempdir}"
    _do_initial_indexing

    # Compare expected browsing results with actual results
    ls -1 "$tempdir/"{audio,videos,images}/* | sort > "${expected_files_log}"
    # Sync after writing to file
    sync
    "${TESTLIBDIR}/grilo-local-storage-tracker-browse" | tee "${browse_output_log}" | \
        sed -n 's|\s\+URL: file://\(.*\)|\1|p' | sort > "${browsed_files_log}"
    # Sync after writing to file
    sync
    # Test whether all the files are browsed
    if ! diff -u "${expected_files_log}" "${browsed_files_log}"; then
        whine "Some files aren't being browsed by grilo!"
        ret=1
    # Test whether audio metadata is extracted properly
    elif [[ $(grep -e "${expected_output_string}" "${browse_output_log}" | wc -l) != 4 ]]; then
        whine "Grilo isn't extracting audio metadata properly!"
        ret=1
    fi
    _kill_monitor
    _reset_tracker_config
    set +x
    return $ret
}

trap "test_failure" ERR

src_test_pass <<-EOF
test_audio_indexing
test_playlist_indexing
test_video_indexing
test_image_indexing
test_document_indexing
test_recursive_indexing
test_playlist_contents_indexing
test_journal_disabled
test_monitoring_add
test_monitoring_delete
test_monitoring_modify
test_query_metadata_info
test_grilo_local_storage_tracker
EOF

test_success
