/* vim: set sts=4 sw=4 et :
 *
 * Mounts any removable devices that are attached to the system, and umounts
 * them on SIGINT/SIGTERM.
 *
 * Usage:
 * 1. Compile it:
 *  gcc $(pkg-config --libs --cflags gio-2.0) -o automount-removable-device{,.c}
 * 2. Run it:
 *  ./automount-removable-device
 * 3. Monitor the output of the command `mount` to keep track of what's mounted
 * 4. Insert USB device, and notice that it gets automatically mounted
 *
 * To unmount all devices mounted by this program, send SIGINT or SIGTERM to it.
 */

#include <gio/gio.h>
#include <glib.h>
#include <glib-unix.h>

static GMainLoop *loop;

static void
unmount_finished (GMount       *mount,
                  GAsyncResult *res,
                  gpointer      user_data)
{
    char *name;
    GError *error = NULL;

    name = g_mount_get_name (mount);
    if (!g_mount_unmount_with_operation_finish (mount, res, &error)) {
        g_warning ("Unmount of '%s' failed: %s", name, error->message);
        g_error_free (error);
    } else {
        g_debug ("Unmount of '%s' completed", name);
    }

    g_free (name);
    g_object_unref (mount);
}

static void
umount_each (GMount   *mount,
             gpointer  user_data)
{
    char *name;

    name = g_mount_get_name (mount);
    g_debug ("Umounting '%s'", name);
    if (!g_mount_can_unmount (mount))
        g_warning ("Can't umount %s", name);
    g_mount_unmount_with_operation (mount,
                                    G_MOUNT_UNMOUNT_NONE,
                                    NULL, /* GMountOperation */
                                    NULL, /* GCancellable */
                                    (GAsyncReadyCallback) unmount_finished,
                                    NULL);
    g_free (name);
}

static gboolean
eject_all (GList **all_mounts)
{
    if (!all_mounts)
        /* Nothing mounted? */
        return G_SOURCE_CONTINUE;

    g_debug ("'%u' volumes mounted", g_list_length (*all_mounts));
    /* Umount ALL the things */
    g_list_foreach (*all_mounts, (GFunc) umount_each, NULL);
    g_list_free (*all_mounts);
    *all_mounts = NULL;
    g_main_loop_quit (loop);

    return G_SOURCE_CONTINUE;
}

static void
volume_mounted_cb (GVolume       *volume,
                   GAsyncResult  *res,
                   GList        **all_mounts)
{
    GMount *mount;
    GFile *location;
    gchar *mount_path;
    gchar *volume_name;
    GError *error = NULL;

    volume_name = g_volume_get_name (volume);

    if (!g_volume_mount_finish (volume, res, &error))
        g_error ("Unable to mount volume '%s': %s",
                 volume_name, error->message);

    mount = g_volume_get_mount (volume);
    if (!mount)
        g_error ("Volume '%s' failed to automount!", volume_name);
    location = g_mount_get_default_location (mount);
    mount_path = g_file_get_path (location);

    g_debug ("Mounting of volume '%s' succeeded at '%s'",
             volume_name, mount_path);
    *all_mounts = g_list_prepend (*all_mounts, mount);

out:
    g_object_unref (location);
    g_object_unref (volume);
    g_free (mount_path);
}

static void
volume_added_cb (GVolumeMonitor  *volume_monitor,
                 GVolume         *volume,
                 GList          **all_mounts)
{
    gchar *volume_name;

    volume_name = g_volume_get_name (volume);

    g_debug ("Mounting volume, '%s'", volume_name);

    if (!g_volume_can_mount (volume)) {
        g_debug ("The volume '%s' inserted cannot be mounted", volume_name);
        goto out;
    }

    if (!g_volume_should_automount (volume)) {
        g_debug ("The volume '%s' inserted will not be automounted", volume_name);
        goto out;
    }

    g_volume_mount (volume,
                    G_MOUNT_MOUNT_NONE,
                    NULL, /* GMountOperation */
                    NULL, /* GCancellable */
                    (GAsyncReadyCallback) volume_mounted_cb,
                    all_mounts);
out:
    g_free (volume_name);
}

static void
volume_removed_cb (GVolumeMonitor  *volume_monitor,
                   GVolume         *volume,
                   GList          **all_mounts)
{
    gchar *volume_name;

    volume_name = g_volume_get_name (volume);
    g_debug ("Volume '%s' was removed", volume_name);
    g_free (volume_name);
}

int
main (int   argc,
      char *argv[])
{
    GVolumeMonitor *volume_monitor;
    /* Keep track of all the mounts to umount them on exit */
    GList *all_mounts = NULL;

    volume_monitor = g_volume_monitor_get ();
    loop = g_main_loop_new (NULL, FALSE);

    g_unix_signal_add (SIGINT, (GSourceFunc) eject_all, &all_mounts);
    g_unix_signal_add (SIGTERM, (GSourceFunc) eject_all, &all_mounts);
    g_signal_connect (volume_monitor, "volume-added",
                      G_CALLBACK (volume_added_cb), &all_mounts);
    g_signal_connect (volume_monitor, "volume-removed",
                      G_CALLBACK (volume_removed_cb), &all_mounts);
/*    g_signal_connect (volume_monitor, "volume-changed",
                      G_CALLBACK (volume_changed_cb), NULL);*/
    g_main_loop_run (loop);
    g_object_unref (volume_monitor);
    g_main_loop_unref (loop);

    return 0;
}
